program tri_deee_zwai;

uses
  VGA256, Graph, Crt;

{----------------------------------------------------------------------------}
const
  D  = 410;       { Vzdalenost pocatku od prumetny }
  SX : integer = 320;       { X souradnice pocatku na obrazovce }
  SY : integer = 225;       { Y souradnice pocatku na obrazovce }
  cNBodu = 2000;    { Maximalni pocet bodu v poli bodu }
  cBackCol = Black;
type
  tCoords = array [1..4] of Real;           { Souradnice X,Y,Z,W }
  tMatice = array [1..4,1..4] of Real;      { Matice 4*4 }

  {---------Transformace---------}
  pTrans = ^tTrans;
  tTrans = object
    M: tMatice;  { Vlastni matice transformace }

    {Metody:}
    constructor Init;
    procedure Jednotkova;
    procedure RotaceX (Alfa: Real);
    procedure RotaceY (Alfa: Real);
    procedure RotaceZ (Alfa: Real);

    procedure Rotace2X (X,Y,Z,Alfa: Real);
{    procedure Rotace2Y (X,Y,Z,Alfa: Real);
    procedure Rotace2Z (X,Y,Z,Alfa: Real);
}
    procedure Posun (DX, DY, DZ: Real);
    procedure Sloz (T1, T2: pTrans);
    procedure Sloz2 (T2: pTrans);
    procedure Write;

   { procedure SetMat (X,Y: Byte; Value: Real);}
   { function  GetMat (X,Y: Byte): Real;}
  end;


  {---------Bod----------}
  pBod = ^tBod;
  tBod = object
    Col: Byte;
    Poz: tCoords;
    X1,Y1, X2, Y2{, X3, Y3}: Integer;

    constructor Init (_Col: Byte);
    procedure Write;
    procedure Transform (T: pTrans);
    procedure Draw;
    procedure Clear;
    procedure SetAllPoz (X,Y,Z: Real);
    function GetX: Real;
    function GetY: Real;
    function GetZ: Real;

  end;
  {--------PoleBodu--------}

  tPoleBodu = array [1..cNBodu] of pBod;

  {---------Body-----------}
  pBody = ^tBody;
  tBody = object
    Pocet    : Integer;
    Col      : Byte;
    PoleBodu : tPoleBodu;

    constructor Init (_Pocet: Integer; _Col: Byte);
    procedure Draw;
    procedure Clear;
    procedure TransformAll (T: pTrans);
    procedure Transform (T: pTrans; First,Last: Integer);
    procedure Usecka (X1,Y1,Z1,X2,Y2,Z2: Real);
    procedure Usecka2(X1,Y1,Z1,X2,Y2,Z2: Real; First,Last: Integer);
    procedure Kruznice (X, Y, Z, R: Real; First, Last: Integer);
    procedure Koule (X, Y, Z, R: Real; First, Last, Kr: Integer);
    procedure Krychle (X, Y, Z, Hrana: Real; First,Last: Integer);
    procedure Obarvi (_Col: Byte; First,Last: Integer);
    procedure Prechod (Body2: pBody; F1, L, F2, Fazi: Integer);
    procedure Rozptyl (F,L: Integer);
    procedure Plocha (X,Y,Z, Hrana: Real; F,L: Integer);
    destructor Done;
  end;

var
  ActivePage: Byte;


{****************************************************************************}
{************              Ostatni pomocne funkce               *************}
{****************************************************************************}
procedure ShowAxes;
var
  Size: Integer;

begin
  Size := 20;
  SetColor (LightGray);
  Line (SX-Size, SY, SX+Size, SY);
  Line (SX, SY-Size, SX, SY+Size);
end;
{--------------------------------------------------------------------Init---}
procedure Init;
var
  grDriver: Integer;
  grMode: Integer;
  ErrCode: Integer;
begin
  grDriver := Detect;
  InitGraph(grDriver, grMode,'');
  ErrCode := GraphResult;
  if ErrCode <> grOk then begin
    Writeln ('Chyba grafiky: ', GraphErrorMsg(ErrCode));
    Halt;
  end;
  SetGraphMode (1);

  ActivePage := 0;
  SetActivePage (0);
  SetVisualPage (0);

  SX := GetMaxX div 2;
  SY := GetMaxY div 2;

end; {Init}

{------------------------------------------------------------------Finish---}
procedure Finish;
begin
  CloseGraph;
  ClrScr;
  Halt;
end; {Finish}

procedure PrepareGraphic;
begin
  ActivePage := (ActivePage + 1) mod 2;
  SetActivePage (ActivePage);
end;

procedure ShowGraphic;
begin
  SetVisualPage (ActivePage);
end;

{****************************************************************************}
{*********                       Transformace                       *********}
{****************************************************************************}

{--------------------------------------------------------------------Init---}
constructor tTrans.Init;
begin { tTrans.Init}
  Jednotkova;
end;  { tTrans.Init}

{--------------------------------------------------------------Jednotkova---}
{ nastavi transformacni matici na jednotkovou matici }
procedure tTrans.Jednotkova;
var
  r,s: Byte;
begin
  for r := 1 to 4 do
    for s := 1 to 4 do
      M [r,s] := 0;

  for r := 1 to 4 do
    M [r,r] := 1;
end;
{-----------------------------------------------------------------RotaceX---}
procedure tTrans.RotaceX (Alfa: Real);
var
  S, C: Real;
begin
  Alfa := (Alfa * Pi)/180;
  S := sin (Alfa);
  C := cos (Alfa);
  Jednotkova;
  M [2,2] := C;
  M [2,3] := S;
  M [3,2] := -S;
  M [3,3] := C;
end; {RotaceX}

{-----------------------------------------------------------------Rotace2X---}
procedure tTrans.Rotace2X (X,Y,Z,Alfa: Real);
var
  S, C: Real;
  R: pTrans;
begin
  R := new (pTrans, Init);
  Posun (-X, -Y, -Z);
  R^.RotaceX (Alfa);
  Sloz2 (R);
  R^.Posun (X,Y,Z);
  Sloz2 (R);
  Dispose (R);
end; {Rotace2X}

{-----------------------------------------------------------------RotaceY---}
procedure tTrans.RotaceY (Alfa: Real);
var
  S, C: Real;
begin
  Alfa := (Alfa * Pi)/180;
  S := sin (Alfa);
  C := cos (Alfa);
  Jednotkova;
  M [1,1] := C;
  M [1,3] := S;
  M [3,1] := -S;
  M [3,3] := C;
end; {RotaceY}

{-----------------------------------------------------------------RotaceZ---}
procedure tTrans.RotaceZ (Alfa: Real);
var
  S, C: Real;
begin
  Alfa := (Alfa * Pi)/180;
  S := sin (Alfa);
  C := cos (Alfa);
  Jednotkova;
  M [1,1] := C;
  M [1,2] := S;
  M [2,1] := -S;
  M [2,2] := C;
end; {RotaceZ}

procedure tTrans.Posun (DX, DY, DZ: Real);
begin
  Jednotkova;
  M [4,1] := DX;
  M [4,2] := DY;
  M [4,3] := DZ;
end;
{--------------------------------------------------------------------Sloz---}
{ Nastavi transformacni matici na slozeni transformaci T1 a T2}
procedure tTrans.Sloz (T1, T2: pTrans);
var
  i, r, s: Byte;
  Sum: Real;
begin
  Sum := 0;
  for r := 1 to 4 do
    for s := 1 to 4 do begin
      Sum := 0;
      for i := 1 to 4 do
        Sum := Sum + T1^.M[r,i]*T2^.M[i,s];
      M[r,s] := Sum;
    end;
end;

{--------------------------------------------------------------------Sloz---}
{ Nastavi transformacni matici na slozeni teto a T2}
procedure tTrans.Sloz2 (T2: pTrans);
var
  i, r, s: Byte;
  Sum: Real;
  T1: pTrans;
begin
  T1 := new (pTrans, Init);
  T1^.M := M;
  Sum := 0;
  for r := 1 to 4 do
    for s := 1 to 4 do begin
      Sum := 0;
      for i := 1 to 4 do
        Sum := Sum + T1^.M[r,i]*T2^.M[i,s];
      M[r,s] := Sum;
    end;
{  M := T1^.M;}
  Dispose (T1);
end;

{-------------------------------------------------------------------Write---}
procedure tTrans.Write;
var
  r, s: Byte;
begin
  for r := 1 to 4 do begin
    Writeln;
    for s := 1 to 4 do
      System.Write (' ',M[r,s]:3:0);
  end;
  Writeln;
end;
{****************************************************************************}
{*********                         Bodik                            *********}
{****************************************************************************}
{---------------------------------------------------------------------Init---}
constructor tBod.Init (_Col: Byte);
begin { tBod.Init }
  Col := _Col;
  Poz[1] := 0;
  Poz[2] := 0;
  Poz[3] := 0;
  Poz[4] := 1;

  X1 := -1; Y1 := -1;
  X2 := -1; Y2 := -1;
 { X3 := -1; Y3 := -1;}

end;  { tBod.Init }

{-------------------------------------------------------------------GetXYZ---}
function tBod.GetX;
begin
  GetX := Poz[1];
end;
function tBod.GetY;
begin
  GetY := Poz[2];
end;
function tBod.GetZ;
begin
  GetZ := Poz[3];
end;


{--------------------------------------------------------------------Write---}
procedure tBod.Write;
var
  i: Byte;
begin
  Writeln;
  for i := 1 to 4 do
    System.Write (' ',Poz[i]:3:0);
end; {Write}

{----------------------------------------------------------------Transform---}
{ Provede transformaci T na souradnice bodu }
procedure tBod.Transform (T: pTrans);
var
  s,i: Byte;
  Sum: Real;
  Pom: tCoords;
begin
  for s := 1 to 4 do begin
    Sum := 0;
    for i := 1 to 4 do
      Sum := Sum + Poz[i] * T^.M[i,s];
    Pom[s] := Sum;
  end;
  Poz := Pom;
end; {Transform}

{---------------------------------------------------------------------Draw---}
procedure tBod.Draw;
var
  Podil: Real;
  NX, NY: Integer;
begin
{  if (D-Poz[3] <> 0) then}
  Podil := D/(D-Poz[3]);
{  else Podil := 1;}

  NX := Round (Poz[1] * Podil);
  NY := - Round (Poz[2] * Podil);

{
  Poz[1] := X.Coords[1]*D/(D-X.Coords[3]);
  X.Coords[2] := X.Coords[2]*D/(D-X.Coords[3]);

  Circle (Round (320+X.Coords[1]), Round (240-X.Coords[2]), 3);
}
{  SetColor (Col);}
{  Circle (SX+X, SY+Y, 1);}
{  line (SX+OX, SY+OY,SX+NX, SY+NY);}

{  PutPixel (SX+NX, SY+NY, Col);
  PutPixel (SX+X1, SY+Y1, Col);}
  {
  SetColor (Col);
  line (SX+X1, SY+Y1,SX+NX, SY+NY);
   }
{  PutPixel (SX+X2, SY+Y2, Col); }
{  else
    PutPixel (SX+NX, SY+NY, Col);}
  PutPixel (SX+NX, SY+NY, Col);

{  X3 := X2;
  Y3 := Y2;
 }
  X2 := X1;
  Y2 := Y1;

  X1 := NX;
  Y1 := NY;
{   LineTo (SX+X, SY+Y);}
end; {Draw}

procedure tBod.Clear;
begin
{  SetColor (Black);}
{  PutPixel (SX+X2, SY+Y2, Black);
  PutPixel (SX+X3, SY+Y3, Black);}
{  Line (SX+X2, SY+Y2, SX+X3, SY+Y3);}

  PutPixel (SX+X2, SY+Y2, Black);
end;

{----------------------------------------------------------------SetAllPoz---}
procedure tBod.SetAllPoz (X,Y,Z: Real);
begin
  Poz [1] := X;
  Poz [2] := Y;
  Poz [3] := Z;
end; {SetAllPoz}

{****************************************************************************}
{*********                         Bodici                           *********}
{****************************************************************************}
{---------------------------------------------------------------------Init---}
constructor tBody.Init (_Pocet: Integer; _Col: Byte);
var
  i: Integer;
begin { tBody.Init }
  Col := _Col;
  Pocet := _Pocet;

  for i := 1 to Pocet do
    PoleBodu [i] := new (pBod, Init (Col));
end;  { tBody.Init }

{---------------------------------------------------------------------Done---}
destructor tBody.Done;
var
  i: Integer;
begin
  for i := 1 to Pocet do
    Dispose (PoleBodu [i]);
end; {Done}

{---------------------------------------------------------------------Draw---}
procedure tBody.Draw;
var
  i: Integer;
begin
{  for i := 1 to Pocet do
    PoleBodu [i]^.Clear; }

  for i := 1 to Pocet do
    PoleBodu [i]^.Draw;
end; {Draw}

{--------------------------------------------------------------------Clear---}
procedure tBody.Clear;
var
  i: Integer;
begin
  for i := 1 to Pocet do
    PoleBodu [i]^.Clear;
end; {Draw}

{-------------------------------------------------------------TransformAll---}
procedure tBody.TransformAll (T: pTrans);
var
  i: Integer;
begin
  for i := 1 to Pocet do
    PoleBodu [i]^.Transform (T);
end; {TransformAll}

{----------------------------------------------------------------Transform---}
procedure tBody.Transform (T: pTrans; First,Last: Integer);
var
  i: Integer;
begin
  for i := First to Last do
    PoleBodu [i]^.Transform (T);
end; {Transform}

{-------------------------------------------------------------------Usecka---}
procedure tBody.Usecka (X1,Y1,Z1,X2,Y2,Z2: Real);
var
  i : Integer;
  DX,DY,DZ: Real;
begin
  DX := (X2-X1)/(Pocet-1);
  DY := (Y2-Y1)/(Pocet-1);
  DZ := (Z2-Z1)/(Pocet-1);

  for i := 0 to Pocet-1 do
    PoleBodu[i+1]^.SetAllPoz (X1+i*DX, Y1+i*DY, Z1+i*DZ);
end; {Usecka}

{------------------------------------------------------------------Usecka2---}
procedure tBody.Usecka2 (X1,Y1,Z1,X2,Y2,Z2: Real; First,Last: Integer);
var
  i : Integer;
  DX,DY,DZ: Real;
  Count : Integer;
begin
  Count := Last - First + 1;
  DX := (X2-X1)/(Count-1);
  DY := (Y2-Y1)/(Count-1);
  DZ := (Z2-Z1)/(Count-1);

  for i := 0 to Count-1 do
    PoleBodu[First+i]^.SetAllPoz (X1+i*DX, Y1+i*DY, Z1+i*DZ);
end; {Usecka2}

{-----------------------------------------------------------------Kruznice---}
procedure tBody.Kruznice (X, Y, Z, R: Real; First, Last: Integer);
var
  Count,i: Integer;
  A: Real;        {Uhel pootoceni}
  RA: pTrans;
begin
  Count := Last - First + 1;
  A := 360 / Count;
  RA  := new (pTrans, Init);

  for i := 0 to Count-1 do begin
    PoleBodu [First+i]^.SetAllPoz (0, R, Z);
    RA^.RotaceZ (i*A);
    PoleBodu [First+i]^.Transform (RA);
  end;

  { Mame body na kruznici a ted uz je jenom presuneme do X, Y, Z}

  RA^.Posun (X, Y, Z);
  Transform (RA, First, Last);

  Dispose (RA);
end; {Kruznice}

{-----------------------------------------------------------------Koule---}
procedure tBody.Koule (X, Y, Z, R: Real; First, Last, Kr: Integer);
var
  Count,i,j: Integer;
  SY: Real;
  A: Real;        {Uhel pootoceni}
  RA: pTrans;
  F, Add: Integer;
begin
  A := 180 / Kr;
  RA  := new (pTrans, Init);
  F := First;
  Add := (Last - First+1) div Kr;

  for i := 1 to Kr do begin
    RA^.RotaceY (i*A*2);
    Kruznice (0, 0, 0, R, F, F+Add-1);
    Transform (RA, F, F+Add-1);
    F := F + Add;
  end;

  RA^.Posun (X, Y, Z);
  Transform (RA, First, Last);
  Dispose (RA);
end; {Koule}
{------------------------------------------------------------------Krychle---}
procedure tBody.Krychle (X, Y, Z, Hrana: Real; First,Last: Integer);
var
  i : Integer;
  DX,DY,DZ: Real;
  Count, Count2 : Integer;
begin
  Hrana := Hrana / 2;
  Count := Last - First + 1;
  Count2 := Count div 12;
  { Predni stena}
  Usecka2 (X-Hrana, Y+Hrana, Z+Hrana,
           X+Hrana, Y+Hrana, Z+Hrana, First, First+Count2-1);
  First := First + Count2;

  Usecka2 (X+Hrana, Y+Hrana, Z+Hrana,
           X+Hrana, Y-Hrana, Z+Hrana, First, First+Count2-1);
  First := First + Count2;

  Usecka2 (X+Hrana, Y-Hrana, Z+Hrana,
           X-Hrana, Y-Hrana, Z+Hrana, First, First+Count2-1);
  First := First + Count2;

  Usecka2 (X-Hrana, Y-Hrana, Z+Hrana,
           X-Hrana, Y+Hrana, Z+Hrana, First, First+Count2-1);
  First := First + Count2;


  {Zadni stena}
  Usecka2 (X-Hrana, Y+Hrana, Z-Hrana,
           X+Hrana, Y+Hrana, Z-Hrana, First, First+Count2-1);
  First := First + Count2;

  Usecka2 (X+Hrana, Y+Hrana, Z-Hrana,
           X+Hrana, Y-Hrana, Z-Hrana, First, First+Count2-1);
  First := First + Count2;

  Usecka2 (X+Hrana, Y-Hrana, Z-Hrana,
           X-Hrana, Y-Hrana, Z-Hrana, First, First+Count2-1);
  First := First + Count2;

  Usecka2 (X-Hrana, Y-Hrana, Z-Hrana,
           X-Hrana, Y+Hrana, Z-Hrana, First, First+Count2-1);
  First := First + Count2;

  {Pricky}
  Usecka2 (X-Hrana, Y+Hrana, Z-Hrana,
           X-Hrana, Y+Hrana, Z+Hrana, First, First+Count2-1);
  First := First + Count2;

  Usecka2 (X+Hrana, Y+Hrana, Z-Hrana,
           X+Hrana, Y+Hrana, Z+Hrana, First, First+Count2-1);
  First := First + Count2;

  Usecka2 (X+Hrana, Y-Hrana, Z-Hrana,
           X+Hrana, Y-Hrana, Z+Hrana, First, First+Count2-1);
  First := First + Count2;

  Usecka2 (X-Hrana, Y-Hrana, Z-Hrana,
           X-Hrana, Y-Hrana, Z+Hrana, First, First+Count2-1);
  First := First + Count2;

end;

{-------------------------------------------------------------------Obarvi---}
procedure tBody.Obarvi (_Col: Byte; First,Last: Integer);
var
  i: integer;
begin
  for i := First to Last do
    PoleBodu[i]^.Col := _Col;
end; {Obarvi}

{------------------------------------------------------------------Prechod---}
procedure tBody.Prechod (Body2: pBody; F1, L, F2, Fazi: Integer);
var
  i: Integer;
  X, Y, Z, DX, DY, DZ: Real;
begin
  for i := F1 to L do begin
    X := PoleBodu[i]^.GetX;
    Y := PoleBodu[i]^.GetY;
    Z := PoleBodu[i]^.GetZ;

    DX := Body2^.PoleBodu[i]^.GetX-X;
    DY := Body2^.PoleBodu[i]^.GetY-Y;
    DZ := Body2^.PoleBodu[i]^.GetZ-Z;

    if (Fazi < 1) then Fazi := 1;

    PoleBodu[i]^.SetAllPoz (X+DX/Fazi, Y+DY/Fazi, Z+DZ/Fazi);
  end;
end;

{------------------------------------------------------------------Rozptyl---}
procedure tBody.Rozptyl (F,L: Integer);
var
  i: Integer;
begin
  for i := F to L do
    PoleBodu[i]^.SetAllPoz (
      PoleBodu[i]^.GetX+PoleBodu[i]^.GetX/(Random (55)+1),
      PoleBodu[i]^.GetY+PoleBodu[i]^.GetY/(Random (55)+1),
      PoleBodu[i]^.GetZ+PoleBodu[i]^.GetZ/(Random (55)+1));
end;

{-------------------------------------------------------------------Plocha---}
procedure tBody.Plocha (X,Y,Z, Hrana: Real; F,L: Integer);
var
  SX, SZ, Add : Real;
  Count, VelHrany, i, j: Integer;
begin
  Count := L - F + 1;
  VelHrany := Trunc (sqrt (Count));
  SX := X - Hrana/2;
  SZ := Z - Hrana/2;
  Add := Hrana/(VelHrany-1);

  for i := 0 to VelHrany-1 do
    for j := 0 to VelHrany-1 do
      PoleBodu[F+i*VelHrany+j]^.SetAllPoz (SX+j*Add, Y, SZ+i*Add);
end;


{****************************************************************************}
(*
procedure HandleKey (C: Char);

begin
  if C = #0 then C := ReadKey;

  if C = #27 then Finish;
  if C = 'q' then
    P^.TransformAll (R4);


  if C = ' ' then begin
    P^.TransformAll (R5);

  end;
end;
*)

var
  RX, RY, RZ, R3, R4, R5: pTrans;
  P, P2: pBody;
  i: Integer;
  Z, ER: Real;

begin
  Init;

  RX  := new (pTrans, Init);
  RY  := new (pTrans, Init);
  RZ  := new (pTrans, Init);
  R3  := new (pTrans, Init);
  R4  := new (pTrans, Init);
  R5  := new (pTrans, Init);


  RX^.RotaceX (5);
  RY^.RotaceY (1);
  RZ^.RotaceZ (2);

  R3^.Sloz (RX, RZ);
  R4^.Sloz (R3, RY);

  P := new (pBody, Init (900,White));
  P2:= new (pBody, Init (900,White));

  P^.Koule (0,0,0,10,1,900, 30);

  for i := 1 to 20 do begin
    PrepareGraphic;
    P^.Clear;
    P^.Draw;
    P^.Rozptyl (1,900);
    ShowGraphic;
    Delay (30);
    if (KeyPressed) then Finish;
  end;

  {-------------------------------}
  for i := 1 to 50 do begin
    PrepareGraphic;
    P^.Clear;
    P^.Draw;
    P^.TransformAll (R4);
    ShowGraphic;
    Delay (30);
    if (KeyPressed) then Finish;
  end;

  P2^.Koule (0,0,0,100,1,900,30);

  for i := 150 downto 1 do begin
    PrepareGraphic;
    P^.Clear;
    P^.Draw;
    P^.Prechod (P2, 1,900,1,i);
    P^.TransformAll (R4);

    ShowGraphic;
    Delay (10);
    if (KeyPressed) then Finish;
  end;

  for i := 1 to 50 do begin
    PrepareGraphic;
    P^.Clear;
    P^.Draw;
    P^.TransformAll (R4);
    ShowGraphic;
    Delay (30);
    if (KeyPressed) then Finish;
  end;

  P2^.Plocha (0,0,0,300,1,900);

  for i := 150 downto 1 do begin
    PrepareGraphic;
    P^.Clear;
    P^.Draw;
    P^.Prechod (P2, 1,900,1,i);
    P^.TransformAll (R4);

    ShowGraphic;
    Delay (10);
    if (KeyPressed) then Finish;
  end;

  for i := 1 to 70 do begin
    PrepareGraphic;
    P^.Clear;
    P^.Draw;
    P^.TransformAll (R4);
    ShowGraphic;
    Delay (30);
    if (KeyPressed) then Finish;
  end;

  P2^.Kruznice (0,0,0,40,1,300);
  P2^.Kruznice (0,0,0,100,301,600);
  P2^.Kruznice (0,0,0,130,601,900);

  for i := 150 downto 1 do begin
    PrepareGraphic;
    P^.Clear;
    P^.Draw;
    P^.Prechod (P2, 1,900,1,i);
    P^.TransformAll (R4);

    ShowGraphic;
    Delay (10);
    if (KeyPressed) then Finish;
  end;

  for i := 1 to 80 do begin
    PrepareGraphic;
    P^.Clear;
    P^.Draw;
    P^.TransformAll (R4);
    ShowGraphic;
    Delay (30);
    if (KeyPressed) then Finish;
  end;

  R5^.RotaceX (90);

  P2^.Koule (0,0,0,40,1,300,10);
  P2^.Koule (0,0,0,60,301,600,10);
  P2^.Koule (0,0,0,80,601,900,10);

  P2^.Transform (R5,301, 600);
  R5^.RotaceY (90);
  P2^.Transform (R5,601, 900);



  for i := 150 downto 1 do begin
    PrepareGraphic;
    P^.Clear;
    P^.Draw;
    P^.Prechod (P2, 1,900,1,i);
    P^.TransformAll (R4);

    ShowGraphic;
    Delay (10);
    if (KeyPressed) then Finish;
  end;

  for i := 1 to 150 do begin
    PrepareGraphic;
    P^.Clear;
    P^.Draw;
    P^.TransformAll (R4);
    ShowGraphic;
    Delay (30);
    if (KeyPressed) then Finish;
  end;

  P2^.Krychle (0,0,0,100,1,900);

  for i := 150 downto 1 do begin
    PrepareGraphic;
    P^.Clear;
    P^.Draw;
    P^.Prechod (P2, 1,900,1,i);
    P^.TransformAll (R4);

    ShowGraphic;
    Delay (10);
    if (KeyPressed) then Finish;
  end;

  for i := 1 to 5000 do begin
    PrepareGraphic;
    P^.Clear;
    P^.Draw;
    P^.TransformAll (R4);
    ShowGraphic;
    Delay (30);
    if (KeyPressed) then Finish;
  end;

  Finish;

  Dispose (P, Done);
  Dispose (P2, Done);
  Dispose (R3);
  Dispose (R4);
  Dispose (R5);
  Dispose (RX);
  Dispose (RY);
  Dispose (RZ);
end.

